@@ -82,41 +82,17 @@ class SuicideBoard(chess.Board):
         else:
             return self.is_stalemate() and self._material_balance() == 0
 
-    def is_insufficient_material(self):
-        # Enough material.
-        if self.knights or self.rooks or self.queens or self.kings:
+    def has_insufficient_material(self, color):
+        if self.occupied != self.bishops:
             return False
 
-        # Must have bishops.
-        if not (self.occupied_co[chess.WHITE] & self.bishops and self.occupied_co[chess.BLACK] & self.bishops):
-            return False
-
-        # All pawns must be blocked.
-        w_pawns = self.pawns & self.occupied_co[chess.WHITE]
-        b_pawns = self.pawns & self.occupied_co[chess.BLACK]
-
-        b_blocked_pawns = chess.shift_up(w_pawns) & b_pawns
-        w_blocked_pawns = chess.shift_down(b_pawns) & w_pawns
-
-        if (b_blocked_pawns | w_blocked_pawns) != self.pawns:
-            return False
-
-        turn = self.turn
-        turn = chess.WHITE
-        if any(self.generate_pseudo_legal_moves(self.pawns)):
-            return False
-        turn = chess.BLACK
-        if any(self.generate_pseudo_legal_moves(self.pawns)):
-            return False
-        self.turn = turn
-
-        # Bishop and pawns of each side are on distinct color complexes.
-        if self.occupied_co[chess.WHITE] & chess.BB_DARK_SQUARES == 0:
-            return self.occupied_co[chess.BLACK] & chess.BB_LIGHT_SQUARES == 0
-        elif self.occupied_co[chess.WHITE] & chess.BB_LIGHT_SQUARES == 0:
-            return self.occupied_co[chess.BLACK] & chess.BB_DARK_SQUARES == 0
-        else:
-            return False
+        # In a position with only bishops, check if all our bishops can be
+        # captured.
+        we_some_on_light = self.occupied_co[color] & chess.BB_LIGHT_SQUARES
+        we_some_on_dark = self.occupied_co[color] & chess.BB_DARK_SQUARES
+        they_all_on_dark = not (self.occupied_co[not color] & chess.BB_LIGHT_SQUARES)
+        they_all_on_light = not (self.occupied_co[not color] & chess.BB_DARK_SQUARES)
+        return (we_some_on_light and they_all_on_dark) or (we_some_on_dark and they_all_on_light)
 
     def generate_pseudo_legal_moves(self, from_mask=chess.BB_ALL, to_mask=chess.BB_ALL):
         for move in super().generate_pseudo_legal_moves(from_mask, to_mask):
@@ -228,27 +204,38 @@ class AtomicBoard(chess.Board):
     def is_variant_loss(self):
         return self.kings and not self.kings & self.occupied_co[self.turn]
 
-    def is_insufficient_material(self):
-        if self.is_variant_loss() or self.is_variant_win():
+    def has_insufficient_material(self, color):
+        # Remaining material does not matter if opponent's king is already
+        # exploded.
+        if not (self.occupied_co[not color] & self.kings):
             return False
 
-        if self.pawns or self.queens:
+        # Bare king can not mate.
+        if not (self.occupied_co[color] & ~self.kings):
+            return True
+
+        # As long as the opponent's king is not alone, there is always a chance
+        # their own pieces explode next to it.
+        if self.occupied_co[not color] & ~self.kings:
+            # Unless there are only bishops that cannot explode each other.
+            if self.occupied == self.bishops | self.kings:
+                if not (self.bishops & self.occupied_co[chess.WHITE] & chess.BB_DARK_SQUARES):
+                    return not (self.bishops & self.occupied_co[chess.BLACK] & chess.BB_LIGHT_SQUARES)
+                if not (self.bishops & self.occupied_co[chess.WHITE] & chess.BB_LIGHT_SQUARES):
+                    return not (self.bishops & self.occupied_co[chess.BLACK] & chess.BB_DARK_SQUARES)
             return False
 
+        # Queen or pawn (future queen) can give mate against bare king.
+        if self.queens or self.pawns:
+            return False
+
+        # Single knight, bishop or rook cannot mate against bare king.
         if chess.popcount(self.knights | self.bishops | self.rooks) == 1:
             return True
 
-        # Only knights.
-        if self.occupied == (self.kings | self.knights):
-            return chess.popcount(self.knights) <= 2 and not all(occ & self.knights for occ in self.occupied_co)
-
-        # Only bishops.
-        if self.occupied == (self.kings | self.bishops):
-            # All bishops on opposite colors.
-            if not self.pieces_mask(chess.BISHOP, chess.WHITE) & chess.BB_DARK_SQUARES:
-                return not self.pieces_mask(chess.BISHOP, chess.BLACK) & chess.BB_LIGHT_SQUARES
-            if not self.pieces_mask(chess.BISHOP, chess.WHITE) & chess.BB_LIGHT_SQUARES:
-                return not self.pieces_mask(chess.BISHOP, chess.BLACK) & chess.BB_DARK_SQUARES
+        # Two knights cannot mate against bare king.
+        if self.occupied == self.knights | self.kings:
+            return chess.popcount(self.knights) <= 2
 
         return False
 
@@ -343,7 +330,7 @@ class KingOfTheHillBoard(chess.Board):
     def is_variant_loss(self):
         return self.kings & self.occupied_co[not self.turn] & chess.BB_CENTER
 
-    def is_insufficient_material(self):
+    def has_insufficient_material(self, color):
         return False
 
 
@@ -406,7 +393,7 @@ class RacingKingsBoard(chess.Board):
     def is_variant_win(self):
         return self.is_variant_end() and self.kings & self.occupied_co[self.turn] & chess.BB_RANK_8
 
-    def is_insufficient_material(self):
+    def has_insufficient_material(self, color):
         return False
 
     def status(self):
@@ -457,7 +444,8 @@ class HordeBoard(chess.Board):
     def is_variant_win(self):
         return self.occupied and not self.occupied_co[not self.turn]
 
-    def is_insufficient_material(self):
+    def has_insufficient_material(self, color):
+        # TODO: Could detect some cases where the Horde can no longer mate.
         return False
 
     def status(self):
@@ -477,6 +465,17 @@ class HordeBoard(chess.Board):
         return status
 
 
+class _ThreeCheckBoardState:
+    def __init__(self, board):
+        self.board_state = chess._BoardState(board)
+        self.remaining_checks_w = board.remaining_checks[chess.WHITE]
+        self.remaining_checks_b = board.remaining_checks[chess.BLACK]
+
+    def restore(self, board):
+        self.board_state.restore(board)
+        board.remaining_checks[chess.WHITE] = self.remaining_checks_w
+        board.remaining_checks[chess.BLACK] = self.remaining_checks_b
+
 class ThreeCheckBoard(chess.Board):
 
     aliases = ["Three-check", "Three check", "Threecheck", "Three check chess"]
@@ -489,7 +488,6 @@ class ThreeCheckBoard(chess.Board):
 
     def __init__(self, fen=starting_fen, chess960=False):
         self.remaining_checks = [3, 3]
-        self._root_remaining_checks = None
         super().__init__(fen, chess960=chess960)
 
     def reset_board(self):
@@ -502,23 +500,17 @@ class ThreeCheckBoard(chess.Board):
         self.remaining_checks[chess.WHITE] = 3
         self.remaining_checks[chess.BLACK] = 3
 
-    def push(self, move):
-        if not self._stack:
-            self._root_remaining_checks = self.remaining_checks.copy()
+    def _board_state(self):
+        return _ThreeCheckBoardState(self)
 
+    def push(self, move):
         super().push(move)
         if self.is_check():
             self.remaining_checks[not self.turn] -= 1
 
-    def pop(self):
-        was_in_check = self.is_check()
-        move = super().pop()
-        if was_in_check:
-            self.remaining_checks[self.turn] += 1
-        return move
-
-    def is_insufficient_material(self):
-        return self.occupied == self.kings
+    def has_insufficient_material(self, color):
+        # Any remaining piece can give check.
+        return not (self.occupied_co[color] & ~self.kings)
 
     def set_epd(self, epd):
         # Split into 5 or 6 parts.
@@ -600,8 +592,7 @@ class ThreeCheckBoard(chess.Board):
 
     def copy(self, stack=True):
         board = super().copy(stack=stack)
-        board.remaining_checks[chess.WHITE] = self.remaining_checks[chess.WHITE]
-        board.remaining_checks[chess.BLACK] = self.remaining_checks[chess.BLACK]
+        board.remaining_checks = self.remaining_checks.copy()
         return board
 
     def mirror(self):
@@ -610,13 +601,17 @@ class ThreeCheckBoard(chess.Board):
         board.remaining_checks[chess.BLACK] = self.remaining_checks[chess.WHITE]
         return board
 
-    def root(self):
-        board = super().root()
-        if self._stack:
-            board.remaining_checks[chess.WHITE] = self._root_remaining_checks[chess.WHITE]
-            board.remaining_checks[chess.BLACK] = self._root_remaining_checks[chess.BLACK]
-        return board
 
+class _CrazyhouseBoardState:
+    def __init__(self, board):
+        self.board_state = chess._BoardState(board)
+        self.pockets_w = board.pockets[chess.WHITE].copy()
+        self.pockets_b = board.pockets[chess.BLACK].copy()
+
+    def restore(self, board):
+        self.board_state.restore(board)
+        board.pockets[chess.WHITE] = self.pockets_w.copy()
+        board.pockets[chess.BLACK] = self.pockets_b.copy()
 
 class CrazyhousePocket:
 
@@ -663,7 +658,6 @@ class CrazyhouseBoard(chess.Board):
 
     def __init__(self, fen=starting_fen, chess960=False):
         self.pockets = [CrazyhousePocket(), CrazyhousePocket()]
-        self._root_pockets = None
         super().__init__(fen, chess960=chess960)
 
     def reset_board(self):
@@ -676,25 +670,13 @@ class CrazyhouseBoard(chess.Board):
         self.pockets[chess.WHITE].reset()
         self.pockets[chess.BLACK].reset()
 
+    def _board_state(self):
+        return _CrazyhouseBoardState(self)
+
     def push(self, move):
-        if not self._stack:
-            self._root_pockets = [pocket.copy() for pocket in self.pockets]
-
-        if move.drop:
-            self.pockets[self.turn].remove(move.drop)
-
         super().push(move)
-
-    def pop(self):
-        move = super().pop()
         if move.drop:
-            self.pockets[self.turn].add(move.drop)
-        elif self.is_capture(move):
-            if self.is_en_passant(move) or chess.BB_SQUARES[move.to_square] & self.promoted:
-                self.pockets[self.turn].remove(chess.PAWN)
-            else:
-                self.pockets[self.turn].remove(self.piece_type_at(move.to_square))
-        return move
+            self.pockets[not self.turn].remove(move.drop)
 
     def _push_capture(self, move, capture_square, piece_type, was_promoted):
         if was_promoted:
@@ -739,16 +721,11 @@ class CrazyhouseBoard(chess.Board):
 
     def is_pseudo_legal(self, move):
         if move.drop and move.from_square == move.to_square:
-            if move.drop == chess.KING:
-                return False
-
-            if chess.BB_SQUARES[move.to_square] & self.occupied:
-                return False
-
-            if move.drop == chess.PAWN and chess.BB_SQUARES[move.to_square] & chess.BB_BACKRANKS:
-                return False
-
-            return self.pockets[self.turn].count(move.drop) > 0
+            return (
+                move.drop != chess.KING and
+                not chess.BB_SQUARES[move.to_square] & self.occupied and
+                not (move.drop == chess.PAWN and chess.BB_SQUARES[move.to_square] & chess.BB_BACKRANKS) and
+                self.pockets[self.turn].count(move.drop) > 0)
         else:
             return super().is_pseudo_legal(move)
 
@@ -784,7 +761,10 @@ class CrazyhouseBoard(chess.Board):
         else:
             return super().parse_san(san)
 
-    def is_insufficient_material(self):
+    def has_insufficient_material(self, color):
+        # In practise no material can leave the game, but this is easy to
+        # implement anyway. Note that bishops can be captured and put onto
+        # a different color complex.
         return (
             chess.popcount(self.occupied) + sum(len(pocket) for pocket in self.pockets) <= 3 and
             not self.pawns and
@@ -840,13 +820,6 @@ class CrazyhouseBoard(chess.Board):
         board.pockets[chess.BLACK] = self.pockets[chess.WHITE].copy()
         return board
 
-    def root(self):
-        board = super().root()
-        if self._stack:
-            board.pockets[chess.WHITE] = self._root_pockets[chess.WHITE].copy()
-            board.pockets[chess.BLACK] = self._root_pockets[chess.BLACK].copy()
-        return board
-
     def status(self):
         status = super().status()
 
